from datetime import date, time, datetime, timedelta
from pytz import timezone
from uuid import uuid4
from os import path
from icalendar import Calendar, Event, vCalAddress, vText

from colloscope.models import *

LOCAL_TZ = "Europe/Paris"


def emailize(nom, prenom=None):
    if prenom is not None:
        return "{}.{}@example.com" \
            .format(
            prenom.replace(" ", "_").lower(),
            nom.replace(" ", "_").lower()
        )
    else:
        return "{}@example.com" \
            .format(nom.replace(" ", "_").lower())


def to_calendar(etudiant, periode):
    p = path.abspath('./static/Base_Calendar.ics')

    with open(p) as f:
        cal = Calendar.from_ical(f.read())

    rotations = Rotation.objects \
        .filter(groupes__membres=etudiant, creneau__periode=periode) \
        .select_related("creneau__periode__classe__lycee") \
        .select_related("creneau__matiere") \
        .select_related("creneau__colleur")

    for rotation in rotations:
        event = Event()

        summary = f"Colle {rotation.creneau.matiere} ({rotation.creneau.colleur})"
        event.add("summary", summary)

        start = rotation.datetime()
        fin = start + rotation.creneau.duree

        event.add("dtstart", start, parameters={"tzid": LOCAL_TZ})
        event.add("dtend", fin, parameters={"tzid": LOCAL_TZ})
        event.add("dtstamp", datetime.now())
        event.add("uid", str(rotation.id))

        event.add("location", f"{rotation.creneau.salle} ({rotation.creneau.periode.classe.lycee})")
        event.add("categories", "COLLE-" + str(rotation.creneau.matiere))

        description = f"Groupes: {','.join(str(groupe) for groupe in rotation.groupes.all())}"
        event.add("description", description)

        organizer = vCalAddress(f"mailto:{emailize(rotation.creneau.colleur.nom)}")
        organizer.params["cn"] = vText(str(rotation.creneau.colleur))
        organizer.params["role"] = vText("Colleur")
        event.add("organizer", organizer)

        for e in rotation.groupe_effectif():
            attendee = vCalAddress(f"mailto:{emailize(e.nom, prenom=e.prenom)}")
            attendee.params["cn"] = vText(str(e))
            attendee.params["role"] = vText("Etudiant")

            event.add("attendee", attendee, encode=0)

        cal.add_component(event)

    return cal
